;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Sugarscape: CW's replication of the model by Epstein & Axtell (1996)
; This version (C) Christopher Watts, 2015
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

extensions [array]

globals [
  sugar-colors
  spice-colors
  group-colors
  
  gini
  summer-in-north?
  
  potential-moves
  num-babies
;  babies
  ages-at-death
  num-starved
  num-killed
  
  trade-prices
  
  max-disease-id
  immune-sys-matches
  disease-freq
  
  last-rng-seed
]

breed [agents agent]
breed [diseases disease]
directed-link-breed [parentages parentage]
undirected-link-breed [friendships friendship]
directed-link-breed [likes like]
directed-link-breed [loans loan]

agents-own [
  metabolic-rate
  spice-metabolic-rate
  vision
  endowment
  spice-endowment
  sugar-stock
  spice-stock
  age
  death-age
  birth-place
  fertility-start
  fertility-end
  fertile?
  sex
  children
  visible-points
  tags
  group
  foresight
  potential-lender?
  potential-borrower?
  max-loan
  genetic-immune-system
  immune-system
  my-diseases
]

diseases-own [
  disease-id ; In case we can't use who e.g. if new diseases emerge during sim.
  code ; Used with agent's immune-system to calculate whether agent still infected
  metabolic-effect ; Increase to infected agent's metabolic rates
  infection-chance ; Chance of an encounter between infected and non-infected resulting in transmission. (Not used.)
]

patches-own [
  in-north?
  max-psugar
  max-pspice
  psugar
  pspice
  pollution
]

loans-own [
  lsugar
  lspice
  ldate
  ldue-date
  linterest-rate
]



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Setting up

to setup
  clear-all
  reset-ticks
  
  setup-rng
  
  set sugar-colors array:from-list (list white (yellow + 4) (yellow + 3) (yellow + 2) yellow )
  set spice-colors array:from-list (list white (orange + 4) (orange + 3) (orange + 2) orange )
  set group-colors array:from-list (list blue red green sky pink lime violet cyan magenta brown)
  
  setup-scape
  if disease? [setup-diseases]
  setup-agents
  set num-babies 0
  set num-starved 0
  set num-killed 0
  set ages-at-death []
  set trade-prices []
  
  initialize-plots
  do-plots
  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Random number generator

to setup-rng
  set last-rng-seed ifelse-value (rng-seed = 0) [new-seed] [rng-seed]
  random-seed last-rng-seed
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Setting up patches

; Scape

to setup-scape
  let cur-type substring scape-type 0 (position ":" scape-type)
  if "Classic" = cur-type [setup-scape-from-strings stop]
  if "Flat Grid" = cur-type [setup-scape-square stop]
  ; Just add your alternative sugarscapes to Scape-Type and call their setup here.
  user-message "Error: Could not identify Scape-Type!" ; Warns you of fatal error.
end

to setup-scape-from-strings
  ; My attempt to replicate the classic Sugarscape design.
  ; I reuse this (flip vertically) for Spice. 
  ; This will produce minor differences from Epstein & Axtell's original.
  ; E.g. different carrying capacity, chance of sustainable population emerging.
  
  let scape reverse (list 
    "00000000000000000011111111122222222233333322222222"
    "00000000000000000011111111222222223333333333222222"
    "00000000000000000111111112222222333333333333332222"
    "00000000000000000111111112222223333333333333333222"
    "00000000000000000111111122222233333333333333333322"
    "00000000000000001111111122222233333334444333333322"
    "00000000000000011111111122222333333444444443333332"
    "00000000000000011111111222222333334444444444333332"
    "00000000000000111111111222223333334444444444333333"
    "00000000000001111111111222223333344444444444433333"
    "00000000000011111111112222223333344444444444433333"
    "00000000000111111111112222223333344444444444433333"
    "00000000001111111111112222223333344444444444433333"
    "00000000011111111111112222223333334444444444333333"
    "00000001111111111111122222223333334444444444333332"
    "00000111111111111111222222223333333444444443333332"
    "11111111111111111112222222223333333334444333333322"
    "11111111111111111122222222222333333333333333333322"
    "11111111111111122222222222222333333333333333333222"
    "11111111112222222222222222222233333333333333332222"
    "11111111222222222222222222222223333333333333222222"
    "11111122222222222222222222222222233333333322222222"
    "11112222222222222222222222222222222222222222222221"
    "11122222222222222222222222222222222222222222222211"
    "11222222222233333333322222222222222222222222222111"
    "12222222223333333333333222222222222222222222211111"
    "12222222333333333333333322222222222222222211111111"
    "22222223333333333333333332222222222222111111111111"
    "22222233333333333333333332222222222221111111111111"
    "22222233333334444333333333222222222211111111111111"
    "22222333333444444443333333222222222211111111111000"
    "22222333334444444444333333222222221111111111110000"
    "22223333334444444444333333222222221111111111000000"
    "22223333344444444444433333222222111111111100000000"
    "22223333344444444444433333222222111111111100000000"
    "22223333344444444444433333222222111111111000000000"
    "22223333344444444444433333222221111111110000000000"
    "22223333334444444444333333222221111111100000000000"
    "22222333334444444444333332222221111111100000000000"
    "22222333333444444443333332222211111111000000000000"
    "22222233333334444333333322222211111110000000000000"
    "22222233333333333333333322222211111110000000000000"
    "22222223333333333333333222222111111110000000000000"
    "12222222333333333333332222221111111110000000000000"
    "12222222223333333333222222221111111110000000000000"
    "11222222222233333322222222211111111110000000000000"
    "11122222222222222222222222111111111100000000000000"
    "11112222222222222222222221111111111100000000000000"
    "11111122222222222222222111111111111100000000000000"
    "11111111222222222222211111111111111000000000000000"
    )
  
  let cur-x 0
  let cur-y 0
  
  foreach scape [
    set cur-x 0
    repeat length ? [
      ask patch cur-x cur-y [
        set psugar (0 + (read-from-string item cur-x ?))
      ]
      ask patch cur-x (max-pycor - cur-y) [
        set pspice (0 + (read-from-string item cur-x ?))
      ]
      set cur-x 1 + cur-x
    ]
    set cur-y 1 + cur-y
  ]
  
  let equator min-pycor + ((max-pycor - min-pycor) / 2)
  ask patches [
    set max-psugar psugar
    set max-pspice pspice
    set in-north? (pycor >= equator) ; horizontal divide between North and South
;    set in-north? (pycor >= (max-pxcor - pxcor)) ; diagonal N-S
    set pollution 1
    recolour-patch
  ]
  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup-scape-square
  set-patch-size 400 / grid-width
  resize-world 0 (grid-width - 1) 0 (grid-width - 1)
  
  ask patches [
    set psugar max-resources
    set pspice max-resources
  ]
  
  let equator min-pycor + ((max-pycor - min-pycor) / 2)
  ask patches [
    set max-psugar psugar
    set max-pspice pspice
    set in-north? (pycor >= equator) ; horizontal divide between North and South
;    set in-north? (pycor >= (max-pxcor - pxcor)) ; diagonal N-S
    set pollution 1
    recolour-patch
  ]
  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to recolour-patch
  if patch-color-scheme = "Sugar" [ set pcolor array:item sugar-colors psugar stop]
  if patch-color-scheme = "Spice" [ set pcolor array:item spice-colors pspice stop]
  if patch-color-scheme = "Pollution" [ set pcolor scale-color brown (log pollution 10) 3 0 stop]
  if patch-color-scheme = "Sugar & Spice" [ 
    set pcolor 
    (ifelse-value (psugar >= pspice) [
        ifelse-value (psugar = pspice) [
          ifelse-value (pxcor mod 2 = pycor mod 2) [array:item sugar-colors psugar] [array:item spice-colors pspice]
        ] 
        [array:item sugar-colors psugar]
      ] 
    [array:item spice-colors pspice]) 
    stop
  ]
  
  if patch-color-scheme = "Pollution" [set pcolor scale-color green (psugar / (1 + log pollution 10)) 0 4]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Setting up agents

to setup-agents
  create-agents Initial-Population [
    setup-agent-attributes
  ]
end

to setup-agent-attributes
  set shape "person"
  set metabolic-rate random-given-mean 1 Mean-Metabolic-Rate
  set spice-metabolic-rate random-given-mean 1 Mean-Metabolic-Rate
  set vision random-given-mean 1 Mean-Vision
  set age ifelse-value Initial-Age-0? [0] [random-uniform 0 ((min-death-age + max-death-age) / 2)]
  set Death-Age random-uniform min-death-age max-death-age
  setup-visible-points
  
  set fertility-start random-uniform min-fertility-start max-fertility-start
  set sex random 2 ; 1 = male; 0 = female
  set fertility-end ifelse-value (sex = 1) [random-uniform min-fertility-end-male max-fertility-end-male] [random-uniform min-fertility-end-female max-fertility-end-female]
  set fertile? ((age >= fertility-start) and (age <= fertility-end))
  set children []
  
  if culture? [
    set tags array:from-list n-values number-of-tags [random 2]
    set group cultural-group
  ]
  
  set foresight random-given-mean 0 Mean-foresight
  
  set endowment random-uniform min-endowment max-endowment
  set spice-endowment random-uniform min-endowment max-endowment
  set sugar-stock endowment
  set spice-stock spice-endowment
  
  set my-diseases no-turtles
  if disease? [
    set my-diseases n-of Agents-Initial-Number-Of-Diseases diseases
    set genetic-immune-system array:from-list n-values immune-system-length [random 2]
    set immune-system array:from-list array:to-list genetic-immune-system
  ]
  
  setup-initial-distribution
  set birth-place patch-here
  recolour-agent
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup-initial-distribution
  if initial-location = "Corner" [
    move-to one-of patches with [pxcor < (max-pxcor / 2) and pycor < (max-pycor / 2) and not any? agents-here]
    stop
  ]
  if initial-location = "2 Group Corners" [
;    ifelse 0 = (group mod 2) [
    ifelse 0 = int (2 * group / number-of-groups) [
      move-to one-of patches with [pxcor < (max-pxcor / 2) and pycor < (max-pycor / 2) and not any? agents-here]
    ]
    [
      move-to one-of patches with [pxcor >= (max-pxcor / 2) and pycor >= (max-pycor / 2) and not any? agents-here]
    ]
    stop
  ]
  if initial-location = "4 Group Corners" [
    ifelse 0 = int (4 * group / number-of-groups) [
      move-to one-of patches with [pxcor < (max-pxcor / 2) and pycor < (max-pycor / 2) and not any? agents-here]
    ]
    [
      ifelse 1 = int (4 * group / number-of-groups) [
        move-to one-of patches with [pxcor < (max-pxcor / 2) and pycor >= (max-pycor / 2) and not any? agents-here]
      ]
      [
        ifelse 2 = int (4 * group / number-of-groups) [
          move-to one-of patches with [pxcor >= (max-pxcor / 2) and pycor >= (max-pycor / 2) and not any? agents-here]
        ]
        [
          move-to one-of patches with [pxcor >= (max-pxcor / 2) and pycor < (max-pycor / 2) and not any? agents-here]
        ]
      ]
    ]
    stop
  ]
  ; Anywhere
  move-to one-of patches with [not any? agents-here]
end


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup-agent-child [mum dad]
;  show (word "born at time " ticks " to " mum " and " dad)
  set shape "person"
  set metabolic-rate [metabolic-rate] of one-of (list mum dad)
  set spice-metabolic-rate [spice-metabolic-rate] of one-of (list mum dad)
  set vision [vision] of one-of (list mum dad)
  set age 0
  set Death-Age random-uniform min-death-age max-death-age
  set birth-place patch-here
  setup-visible-points
  
  set fertility-start random-uniform min-fertility-start max-fertility-start
  set sex random 2 ; 1 = male; 0 = female
  set fertility-end ifelse-value (sex = 1) [random-uniform min-fertility-end-male max-fertility-end-male] [random-uniform min-fertility-end-female max-fertility-end-female]
  set fertile? ((age >= fertility-start) and (age <= fertility-end))
  set children []
  
  if culture? [
    set tags array:from-list n-values number-of-tags [ifelse-value (0 = random 2) [[array:item tags ?] of mum] [[array:item tags ?] of dad] ]
    set group cultural-group
  ]
  
  set foresight [foresight] of one-of (list mum dad)
  
  let gift [int (endowment / 2)] of mum
  let spice-gift [int (spice-endowment / 2)] of mum
  set endowment gift
  set spice-endowment spice-gift
  ask mum [ 
    set children fput myself children
    set sugar-stock sugar-stock - gift
    set spice-stock spice-stock - spice-gift
  ]

  set gift [int (endowment / 2)] of dad
  set spice-gift [int (spice-endowment / 2)] of dad
  set endowment endowment + gift
  set spice-endowment spice-endowment + spice-gift
  ask dad [
    set children fput myself children
    set sugar-stock sugar-stock - gift
    set spice-stock spice-stock - spice-gift
  ]
  set sugar-stock endowment
  set spice-stock spice-endowment
  
  set my-diseases no-turtles
  if disease? [
    set my-diseases (turtle-set )
    set genetic-immune-system array:from-list n-values immune-system-length [ifelse-value (0 = random 2) [[array:item genetic-immune-system ?] of mum] [[array:item genetic-immune-system ?] of dad] ]
    set immune-system array:from-list array:to-list genetic-immune-system
  ]
  
  set num-babies 1 + num-babies
  if link-parentage? [
    create-parentage-from mum
    create-parentage-from dad
  ]
  recolour-agent  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to-report random-uniform [lowest highest]
  report lowest + random (1 + highest - lowest)
end

to-report random-given-mean [lowest given-mean]
  report random-uniform lowest ((given-mean * 2) - lowest)
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup-visible-points
  set visible-points (sentence 
    (list [0 0]) 
    (n-values vision [list 0 (? + 1)]) 
    (n-values vision [list 0 (-1 - ?)]) 
    (n-values vision [list (? + 1) 0]) 
    (n-values vision [list (-1 - ?) 0])
    )
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to recolour-agent
  if agent-color-scheme = "Red" [set color red stop]
  if agent-color-scheme = "Sex" [set color ifelse-value (sex = 1) [blue] [pink] stop]
  if agent-color-scheme = "Vision" [set color ifelse-value (vision >= mean-vision) [blue - 2] [blue + 2] stop]
  if agent-color-scheme = "Metabolic Rate" [set color ifelse-value (metabolic-rate >= mean-metabolic-rate) [red - 2] [red + 2] stop]
  if agent-color-scheme = "Vision & Metabolic Rate" [set color ifelse-value (metabolic-rate >= mean-metabolic-rate) [ifelse-value (vision >= mean-vision) [magenta - 2] [red]] [ifelse-value (vision >= mean-vision) [blue] [magenta + 2]] stop]
  if agent-color-scheme = "Birth-Place" [set color ifelse-value ([in-north?] of birth-place) [green - 2] [orange ] stop]
  if agent-color-scheme = "Age" [set color scale-color red age max-death-age -10 stop]
  if agent-color-scheme = "Fertility" [set color ((ifelse-value (sex = 1) [blue] [red]) + (ifelse-value (age < fertility-start) [3] [ifelse-value (age <= fertility-end) [0] [-3]])) stop]

  if agent-color-scheme = "Sugar-Stock" [set color scale-color lime sugar-stock 50 -10 stop]
  if agent-color-scheme = "Spice-Stock" [set color scale-color magenta spice-stock 50 -10 stop]
  if agent-color-scheme = "Endowment" [set color scale-color green endowment (1.2 * max-endowment) min-endowment stop]
  if agent-color-scheme = "Foresight" [set color ifelse-value (foresight >= mean-foresight) [blue - 2] [blue + 2] stop]
  if agent-color-scheme = "Children" [set color scale-color green (length children) 5 -2 stop]
  if agent-color-scheme = "Cultural Group" [set color (array:item group-colors group) stop]
  if agent-color-scheme = "Welfare" [set color scale-color blue (ifelse-value (0 = (welfare 0 0)) [0] [log (welfare 0 0) 10]) 3 0 stop]
  if agent-color-scheme = "Lending" [set color ifelse-value (any? my-out-loans) [ifelse-value (any? my-in-loans) [yellow] [green]] [ifelse-value (any? my-in-loans) [red] [blue]] stop]
  if agent-color-scheme = "Diseases" [set color ifelse-value (not any? my-diseases) [blue] [ifelse-value (1 < count my-diseases) [red - 2] [red + 1]] stop]
  set color black ; So black agents means none of the above worked!
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Simulation iteration

to go
  if ticks = max-ticks [stop]
  if not any? agents [ stop ]
  tick
  
  set num-babies 0
  set num-starved 0
  set num-killed 0
  set ages-at-death []
  set trade-prices []
  
  ask agents [
    grow-old
    ifelse combat? [
      combat-move
    ]
    [
      ifelse spice? [
        move-to-nearest-best-welfare-patch
      ]
      [
        move-to-nearest-best-patch
      ]
      
    ]
    consume-sugar
    if spice? [consume-spice]
    if trade? [conduct-trade]
    if culture? [transmit-culture]
    if lending? [
      process-debts
      conduct-lending
    ]
    
    if reproduction = "Sexual" [ reproduce-sexually ]
    if reproduction = "Sexual(CW)" [ reproduce-sexually-cw ]
    
    if disease? [
      infect-with-given-chances
      fight-my-diseases
    ]
    
    recolour-agent
  ]
    
  sugar-growback
  if spice? [spice-growback]
  diffuse-pollution  
  
  do-plots
  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Growback

; Sugar

to sugar-growback
  set summer-in-north? 0 = (int (ticks / season-length)) mod 2
  run (word "sugar-growback-" Growback-Method)
end

to sugar-growback-instantly-max
  ifelse seasons? [
    ask patches [
      ifelse in-north? xor summer-in-north? [
        if 0 = (ticks mod Off-Season-Inter-Growback-Time) [
          set psugar max-psugar
          recolour-patch
        ]
      ]
      [
        set psugar max-psugar
        recolour-patch
      ]
    ]
  ]
  [
    set psugar max-psugar
    recolour-patch
  ]
end

to sugar-growback-fixed-rate
  ifelse seasons? [
    ask patches [
      ifelse in-north? xor summer-in-north? [
        if 0 = (ticks mod Off-Season-Inter-Growback-Time) [
          set psugar min (list max-psugar (psugar + Sugar-Growback-Rate))
          recolour-patch
        ]
      ]
      [
        set psugar min (list max-psugar (psugar + Sugar-Growback-Rate))
        recolour-patch
      ]
    ]
  ]
  [
    ask patches [
      set psugar min (list max-psugar (psugar + Sugar-Growback-Rate))
      recolour-patch
    ]
  ]
end

to sugar-growback-poisson
  ifelse seasons? [
    ask patches [
      ifelse in-north? xor summer-in-north? [
        set psugar min (list max-psugar (psugar + random-poisson (Sugar-Growback-Rate / Off-Season-Inter-Growback-Time)))
        recolour-patch
      ]
      [
        set psugar min (list max-psugar (psugar + random-poisson Sugar-Growback-Rate))
        recolour-patch
      ]
    ]
  ]
  [
    ask patches [
      set psugar min (list max-psugar (psugar + random-poisson Sugar-Growback-Rate))
      recolour-patch
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Spice

to spice-growback
  set summer-in-north? 0 = (int (ticks / season-length)) mod 2
  run (word "spice-growback-" Growback-Method)
end

to spice-growback-instantly-max
  ifelse seasons? [
    ask patches [
      ifelse in-north? xor summer-in-north? [
        if 0 = (ticks mod Off-Season-Inter-Growback-Time) [
          set pspice max-pspice
          recolour-patch
        ]
      ]
      [
        set pspice max-pspice
        recolour-patch
      ]
    ]
  ]
  [
    set pspice max-pspice
    recolour-patch
  ]
end

to spice-growback-fixed-rate
  ifelse seasons? [
    ask patches [
      ifelse in-north? xor summer-in-north? [
        if 0 = (ticks mod Off-Season-Inter-Growback-Time) [
          set pspice min (list max-pspice (pspice + spice-Growback-Rate))
          recolour-patch
        ]
      ]
      [
        set pspice min (list max-pspice (pspice + spice-Growback-Rate))
        recolour-patch
      ]
    ]
  ]
  [
    ask patches [
      set pspice min (list max-pspice (pspice + spice-Growback-Rate))
      recolour-patch
    ]
  ]
end

to spice-growback-poisson
  ifelse seasons? [
    ask patches [
      ifelse in-north? xor summer-in-north? [
        set pspice min (list max-pspice (pspice + random-poisson (spice-Growback-Rate / Off-Season-Inter-Growback-Time)))
        recolour-patch
      ]
      [
        set pspice min (list max-pspice (pspice + random-poisson spice-Growback-Rate))
        recolour-patch
      ]
    ]
  ]
  [
    set pspice min (list max-pspice (pspice + random-poisson spice-Growback-Rate))
    recolour-patch
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Pollution

to diffuse-pollution
  ; What happens to a patch's pollution level
  if pollution? [
    if ticks >= pollution-diffusion-on-at [
      if 0 = ticks mod pollution-diffusion-period [
        ask patches [
          set pollution mean [pollution] of neighbors4
;          if pollution = 0 [show (word pollution ", " ticks)]
          recolour-patch
        ]
      ]
    ]
  ]
end

to pollute [amount-collected amount-consumed]
  ; Done by agents when they consume sugar or spice
  if pollution? [
    if ticks >= pollution-on-at-time [
;      if pollution = 0 [show (word pollution ", " ticks)]
      set pollution (pollution + (amount-collected * production-pollution) + (amount-consumed * consumption-pollution))
;      if pollution = 0 [show (word ticks "; " pollution "; " amount-collected "; " amount-consumed)]
    ]
  ] 
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Agent activities

; Ageing

to grow-old
  set age 1 + age
  set fertile? ((age >= fertility-start) and (age <= fertility-end))
  
  if reproduction != "None" [
    if death-age <= age [
      if reproduction = "On death" [replace-dead-agent]
      prepare-for-death
      die
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Movement

to move-to-nearest-best-patch
  set potential-moves (patches at-points visible-points)
  if any? potential-moves [
    move-to min-one-of (potential-moves with-max [psugar / pollution]) [distance myself]
  ]
;  move-to min-one-of (visible-patches with-max [psugar / pollution]) [distance myself]
;  move-to min-one-of ((patches at-points visible-points) with-max [psugar / pollution]) [distance myself]
end

to move-to-nearest-best-welfare-patch
  set potential-moves (patches at-points visible-points) with [not any? agents-here]
  if any? potential-moves [
    move-to min-one-of (potential-moves with-max [agent-welfare / pollution]) [distance myself]
  ]
;  move-to min-one-of (visible-patches with-max [psugar / pollution]) [distance myself]
;  move-to min-one-of ((patches at-points visible-points) with-max [psugar / pollution]) [distance myself]
end

to-report agent-welfare
  ; run as patch, myself = agent
  let sugar-offer psugar
  let spice-offer pspice
  report [welfare sugar-offer spice-offer] of myself
;  let total-metabolic-rate [metabolic-rate + spice-metabolic-rate] of myself
;  report (
;    (psugar + [sugar-stock] of myself) ^ (([metabolic-rate] of myself) / total-metabolic-rate) *
;    (pspice + [spice-stock] of myself) ^ (([spice-metabolic-rate] of myself) / total-metabolic-rate)
;    )
end

to-report welfare [sugar-change spice-change]
  ; run as agent
  let cur-sugar (sugar-change + sugar-stock)
  let cur-spice (spice-change + spice-stock)
  if cur-sugar <= 0 [report 0]
  if cur-spice <= 0 [report 0]
  
  if mean-foresight > 0 [
    set cur-sugar max (list 0 (cur-sugar - (foresight * metabolic-rate)))
    set cur-spice max (list 0 (cur-spice - (foresight * spice-metabolic-rate)))
  ]
  
  ifelse (not culture?) [
    report (
      (cur-sugar ^ (metabolic-rate / (metabolic-rate + spice-metabolic-rate))) *
      (cur-spice ^ (spice-metabolic-rate / (metabolic-rate + spice-metabolic-rate)))
      )
  ]
  [
    let fraction-0 1 - ((sum (map [ifelse-value (? = 0) [1] [0]] (array:to-list tags))) / number-of-tags)
    let weighted-rate-su (metabolic-rate * fraction-0)
    let weighted-rate-sp (spice-metabolic-rate * (1 - fraction-0))
    
    report (
      (cur-sugar ^ (weighted-rate-su / (weighted-rate-su + weighted-rate-sp))) *
      (cur-spice ^ (weighted-rate-sp / (weighted-rate-su + weighted-rate-sp)))
      )
  ]
  
  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Vision

to-report visible-patches
  let given-vision vision
  report [(patch-set self
    (next-patch 1 0 0 given-vision)
    (next-patch 0 1 0 given-vision)
    (next-patch 0 -1 0 given-vision)
    (next-patch -1 0 0 given-vision)
    )] of patch-here
  
end

to-report next-patch [given-dx given-dy given-count given-vision]
  if given-count >= given-vision [report nobody]
  if given-dx + pxcor > max-pxcor [report nobody]
  if given-dx + pxcor < min-pxcor [report nobody]
  if given-dy + pycor > max-pycor [report nobody]
  if given-dy + pycor < min-pycor [report nobody]
  if any? agents-here [report [ (next-patch given-dx given-dy (given-count + 1) given-vision) ] of patch-at given-dx given-dy]
  report [(patch-set self (next-patch given-dx given-dy (given-count + 1) given-vision))] of patch-at given-dx given-dy
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Consumption

to consume-sugar
  let amount-collected psugar
  set sugar-stock sugar-stock + amount-collected
  let amount-consumed min (list (metabolic-rate + effect-of-my-diseases) sugar-stock)
  
  ask patch-here [
    pollute amount-collected amount-consumed
    set psugar psugar - amount-collected
    recolour-patch
  ]
  set sugar-stock sugar-stock - amount-consumed
  if sugar-stock <= 0 [
    if reproduction = "On death" [ replace-dead-agent ]
    prepare-for-death
    set num-starved 1 + num-starved
    die
  ]
end

to consume-spice
  let amount-collected pspice
  set spice-stock spice-stock + amount-collected
  let amount-consumed min (list (spice-metabolic-rate + effect-of-my-diseases) spice-stock)
  
  ask patch-here [
;    pollute amount-collected amount-consumed ; p.127 Spice does not cause pollution.
    set pspice pspice - amount-collected
    recolour-patch
  ]
  set spice-stock spice-stock - amount-consumed
  if spice-stock <= 0 [
    if reproduction = "On death" [ replace-dead-agent ]
    prepare-for-death
    set num-starved 1 + num-starved
    die
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Death and inheritance

to prepare-for-death
  set ages-at-death fput age ages-at-death
  
  ask my-friendships [die]
  ask my-out-parentages [die]
  ask my-in-parentages [die]
  
  if inheritance? [
    let surviving-children filter [is-agent? ?] children
    if 0 < length surviving-children [
      let inheritance max (list 0 (sugar-stock / (length surviving-children)))
      let spice-inheritance max (list 0 (spice-stock / (length surviving-children)))
      foreach surviving-children [
        ask ? [
          set sugar-stock sugar-stock + inheritance
          set spice-stock spice-stock + spice-inheritance
          inherit-loans
        ]
      ]
    ]
  ]
  
  ask my-out-loans [die]
  ask my-in-loans [die]
  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Reproduction

to replace-dead-agent
  hatch-agents 1 [
    setup-agent-attributes 
  ]
end

to reproduce-sexually-CW
  ; My rule: Fertile, sufficiently stocked females reproduce with one fertile male neighbour who has sufficient stock and a spare neighbouring patch
  if fertile? [
    if sex = 0 [ ; Females only. So only one baby per tick.
      if sugar-stock >= endowment [
        if (spice-stock >= spice-endowment) or not spice? [
          let potential-patches neighbors4 with [not any? agents-here]
          let potential-mates []
          let opposite-sex 1 - sex
          let ego self
          ask neighbors4 [
            ask agents-here with [fertile?] [
              if sex = opposite-sex [
                if sugar-stock >= endowment [
                  if (spice-stock >= spice-endowment) or not spice? [
                    if (any? neighbors4 with [not any? agents-here]) or (any? potential-patches) [
                      set potential-mates fput self potential-mates
                    ]
                  ]
                ]
              ]
            ]
          ]
          if 0 < length potential-mates [
            let alter one-of potential-mates ; Could express a preference here - e.g. the richest
            ask alter [
              set potential-patches (patch-set potential-patches (neighbors4 with [not any? agents-here]))
              ask one-of potential-patches [
                sprout-agents 1 [
                  setup-agent-child ego alter
                ]
              ]
            ]
          ]
        ]
      ]
    ]
  ]
end

to reproduce-sexually
  ; Rule S: Reproduce with every fertile neighbour of oppposite sex and sufficient stock who has a spare neighbouring patch
  if fertile? [
;    if sex = 0 [ ; Females only. So only one baby per tick.
      if sugar-stock >= endowment [
        if (spice-stock >= spice-endowment) or not spice? [
          let opposite-sex 1 - sex
          let ego self
          ask neighbors4 [
            ask agents-here with [fertile?] [
              if (sugar-stock >= endowment) [
                if ((spice-stock >= spice-endowment) or not spice?) [
                  if sex = opposite-sex [
                    if [(sugar-stock >= endowment) and ((spice-stock >= spice-endowment) or not spice?)] of ego [ ; Check it.
                      let empty-patches neighbors4 with [not any? agents-here]
                      set empty-patches (patch-set ([neighbors4 with [not any? agents-here]] of ego) empty-patches)
                      if any? empty-patches [
                        let alter self
                        ask one-of empty-patches [
                          sprout-agents 1 [
                            setup-agent-child ego alter
                          ]
                        ]
                      ]
                    ]
                  ]
                ]
              ]
            ]
          ]
        ]
      ]
;    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Culture (tags), groups and combat

to transmit-culture
  let ego self
  let cur-tag 0
  ask neighbors4 with [any? agents-here] [
    ask one-of agents-here [
      set cur-tag random number-of-tags
      array:set tags cur-tag [array:item tags cur-tag] of ego
      set group cultural-group
    ]
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to-report cultural-group
  ; Group based on sum of tag values
  ; For # groups > 2 this makes little sense. Initial # 1s is Binomially distributed, so group sizes not equal!
  report int ((sum array:to-list tags) * number-of-groups / (number-of-tags + 1))
end

to-report hamming-distance [given-alter]
  report sum (map [ifelse-value (?1 = ?2) [0] [1]] (array:to-list tags) ([array:to-list tags] of given-alter))
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Combat

to combat-move
  ; NB: No consideration of age or sex. Baby girls can conquer mature adults.
  let ego-group group
  let targets (patches at-points visible-points) with [not any? agents-here with [ego-group = group]]
  let ego-sugar sugar-stock
  set targets targets with [not any? agents-here with [sugar-stock > ego-sugar]]
  ; No consideration of how many people at target patch. Usually 0 or 1, but could be outnumbered.
  if not any? targets [ move-to-nearest-best-patch stop]
  ; No consideration of retaliation
  ; Given my limited vision, how would I know of the retaliation risk at each patch??
  set targets targets with-max [psugar + (ifelse-value (any? agents-here) [sum [sugar-stock] of agents-here] [0])] ; Best rewards
  move-to min-one-of targets [distance myself]
  ask other agents-here [
    ; Take all their resources
    ask myself [set sugar-stock sugar-stock + [sugar-stock] of myself]
    set sugar-stock 0
    if reproduction = "On death" [ replace-dead-agent ]
    prepare-for-death
    set num-killed 1 + num-killed
    die
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Trade

to conduct-trade
  let ego self
  let trading-partners []
  ask neighbors4 with [any? agents-here] [
    ask agents-here [
      set trading-partners fput self trading-partners
    ]
  ]

;  ; Trade first with agent who offers best price?
;  ifelse marginal-rate-of-substitution >= 1 [
;    set trading-partners sort-by [([marginal-rate-of-substitution] of ?1) < ([marginal-rate-of-substitution] of ?2)] trading-partners
;  ]
;  [
;    set trading-partners sort-by [([marginal-rate-of-substitution] of ?1) > ([marginal-rate-of-substitution] of ?2)] trading-partners
;  ]

  foreach trading-partners [
;    set trade-prices []
    trade-with-partner ?
    
;    if not empty? trade-prices [
;      set-current-plot "Trade Prices"
;      plotxy ticks geomean trade-prices
;    ]
  ]
  
end

to trade-with-partner [alter]
  let still-trading? true
  let ego-mrs 0.0
  let alter-mrs 0.0
  let spice-seller nobody
  let sugar-seller nobody
  let spice-change 0.0
  let sugar-change 0.0
;  set trade-prices []
  
  while [still-trading?] [
    set ego-mrs marginal-rate-of-substitution
    set alter-mrs [marginal-rate-of-substitution] of alter
    if ego-mrs = alter-mrs [stop]
    ifelse (ego-mrs > alter-mrs) [
      set spice-seller self
      set sugar-seller alter
    ]
    [
      set spice-seller alter
      set sugar-seller self
    ]
    let cur-price price ego-mrs alter-mrs
    ifelse cur-price >= 1 [
      set spice-change cur-price
      set sugar-change 1
    ]
    [
      set spice-change 1
      set sugar-change 1 / cur-price
    ]
    
    ; Only accept trade if both better off
    if [(welfare sugar-change (0 - spice-change)) <= (welfare 0 0)] of spice-seller [stop]
    if [(welfare (0 - sugar-change) spice-change) <= (welfare 0 0)] of sugar-seller [stop]
    
    ; Only accept trade if order of mrs not switched
    ifelse ego-mrs > alter-mrs [ ; in which case spice-seller = ego = self; sugar-seller = alter
      if (alt-mean-rate-of-substitution sugar-change (0 - spice-change)) < ([alt-mean-rate-of-substitution (0 - sugar-change) spice-change] of alter) [stop]
    ]
    [
      if (alt-mean-rate-of-substitution (0 - sugar-change) spice-change) > ([alt-mean-rate-of-substitution sugar-change (0 - spice-change)] of alter) [stop]
    ]
    ask sugar-seller [
      set sugar-stock sugar-stock - sugar-change
      set spice-stock spice-stock + spice-change
    ]
    ask spice-seller [
      set sugar-stock sugar-stock + sugar-change
      set spice-stock spice-stock - spice-change
    ]
    set trade-prices fput cur-price trade-prices
    if print-trades? [print (word ticks ": " sugar-seller " sold " sugar-change " sugar to " spice-seller " for " spice-change " spice at a price of " cur-price ". " ([(list sugar-stock spice-stock metabolic-rate spice-metabolic-rate)] of sugar-seller ) ", " ([(list sugar-stock spice-stock metabolic-rate spice-metabolic-rate)] of spice-seller ))]
  ]
end

to-report marginal-rate-of-substitution
  ; How many units of spice is 1 unit of sugar worth?
  report ((spice-stock / spice-metabolic-rate) / (sugar-stock / metabolic-rate))
end

to-report alt-mean-rate-of-substitution [sugar-change spice-change]
  ; How many units of spice is 1 unit of sugar worth?
  report (((spice-stock + spice-change) / spice-metabolic-rate) / ((sugar-stock + sugar-change) / metabolic-rate))
end

to-report price [mrs-a mrs-b]
  ; Price = Geomean of the two agents' marginal rates of substitution
  report sqrt (mrs-a * mrs-b)
end

to-report log-price [mrs-a mrs-b]
  report ((log mrs-a 10) + (log mrs-a 10)) / 2
end

to-report geomean [given-list]
  report 10 ^ (mean (map [log ? 10] given-list))
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Loans

to process-debts
  ask loans with [ldue-date = ticks] [
    ifelse ((([sugar-stock] of end2) > lsugar) and (([spice-stock] of end2) > lspice)) [
      be-repaid
    ]
    [
      be-rescheduled
    ]
  ]
end

to conduct-lending
  determine-lenders-and-borrowers
  ask agents with [potential-borrower?] [
    let potential-creditors []
    ask neighbors4 [
      ask agents-here with [potential-lender?] [
        set potential-creditors fput self potential-creditors
      ]
    ]
    if not empty? potential-creditors [
      set potential-creditors sort potential-creditors
      while [(agent-income > 0) and (sugar-stock < (endowment / 2)) and (not empty? potential-creditors)] [
        make-new-loan (first potential-creditors) self (min (list ([max-loan] of (first potential-creditors)) ((endowment / 2) - sugar-stock))) 0
        set potential-creditors but-first potential-creditors
      ]
    ]
  ]
end

to-report agent-income
  report (max-psugar - metabolic-rate - (sum [lsugar] of my-in-loans))
end

to determine-lenders-and-borrowers
  ask agents [
    ifelse not fertile? [
      ifelse age > fertility-end [
        set max-loan sugar-stock / 2
        set potential-lender? true
        set potential-borrower? false
      ]
      [
        set potential-lender? false
        set potential-borrower? false
        set max-loan 0
      ]
    ]
    [
      set max-loan (sugar-stock - (endowment / 2))
      set potential-lender? (max-loan > 0)
      ifelse agent-income > 0 [
        set potential-borrower? true
      ]
      [
        set potential-borrower? false
      ]
    ]
  ]
    
end

to make-new-loan [lender borrower amount-su amount-sp]
  ask lender [
    create-loan-to borrower [
      set hidden? hide-loan-links?
      set color red
      set lsugar amount-su * (1.0 + (interest-rate / 100.0))
      set lspice amount-sp * (1.0 + (interest-rate / 100.0))
      set ldate ticks
      set ldue-date ticks + loan-period
    ]
    set sugar-stock sugar-stock - amount-su
    set spice-stock spice-stock - amount-sp
  ]
  ask borrower [
    set sugar-stock sugar-stock + amount-su
    set spice-stock spice-stock + amount-sp
  ]
end

to be-repaid
  ask end1 [ ;lender
    set sugar-stock sugar-stock + [lsugar] of myself
    set spice-stock spice-stock + [lspice] of myself
  ]
  ask end2 [ ;borrower
    set sugar-stock sugar-stock - [lsugar] of myself
    set spice-stock spice-stock - [lspice] of myself
  ]
  die
end

to be-rescheduled
  let partial-payment-su min (list lsugar (max (list ([sugar-stock / 2] of end2) 0)))
  let partial-payment-sp min (list lspice (max (list ([spice-stock / 2] of end2) 0)))
  ask end1 [ ;lender
    set sugar-stock sugar-stock + partial-payment-su
    set spice-stock spice-stock + partial-payment-sp
  ]
  ask end2 [ ;borrower
    set sugar-stock sugar-stock - partial-payment-su
    set spice-stock spice-stock - partial-payment-sp
  ]
  set ldue-date ldue-date + loan-period
  set lsugar (lsugar - partial-payment-su) * (1.0 + (interest-rate / 100.0))
  set lspice (lspice - partial-payment-sp) * (1.0 + (interest-rate / 100.0))
  
end

to inherit-loans
  ; run as inheriting child
  ; myself = dying parent
  let num-kids [length children] of myself
  ask [my-out-loans] of myself [
    make-new-loan myself end2 (lsugar / num-kids) (lspice / num-kids)
  ]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Disease

to setup-diseases
  set max-disease-id 0
  repeat number-of-diseases [
    create-diseases 1 [
      ; Using turtles to represent diseases is unnecessary
      ; but makes it easy to reference them.
      set hidden? true ; Disease turtles are ideal types, not actual instances of germs. So hide them.
      set shape "bug" ; In case we ever stop hiding them, this would help us identify them.
      setxy random-xcor random-ycor ; In future, diseases could adopt the mean position of the currently infected...
      
      set disease-id max-disease-id
      set max-disease-id 1 + max-disease-id
      set metabolic-effect (random-given-mean 1 mean-disease-effect) ; Some diseases are worse in their effects than others.
      set code n-values (random-given-mean 1 mean-disease-code-length) [random 2] ; Some diseases will be harder to beat than others.
      set infection-chance 1 ; Currently all 1, but potentially some could be easier to transmit than others. 
                             ; (Could be transmitted aerobically, by touch, fluids etc.)
    ]
  ]
  set immune-sys-matches array:from-list n-values immune-system-length [0]
  
end

to-report effect-of-my-diseases
  if not disease? [report 0]
  if not any? my-diseases [report 0]
  report sum [metabolic-effect] of my-diseases
end

to infect-one-neighbor
  ; One neighbour infected with one disease.
  ; Not currently used.
  if any? my-diseases [
    let ego self
    let potential-targets (turtle-set ([agents-here] of neighbors4)) ; Could use neighbors, in-radius, patch-here...
    if any? potential-targets [
      ask one-of potential-targets [
        set my-diseases (turtle-set my-diseases ([one-of my-diseases] of ego))
      ]
      ; Note: infected with perfect copy of disease. No mutation occurs during transmission.
    ]
  ]
end

to infect-with-given-chances
  ; Part of Rule E, with possible extension (infection-chance).
  if any? my-diseases [
    let infector self
    let potential-targets (turtle-set ([agents-here] of neighbors4)) ; Could use neighbors, in-radius, patch-here...
    ask potential-targets [
;      ask my-diseases [ ; Could alternatively give all diseases a chance to be transmitted.
      let infected self
      ask [one-of my-diseases]  of infector [ ; Following transmission rule E, p.145.
        if infection-chance > (random-float 1) [ ; Can use infection-chance to restrict how many transmitted.
          ask infected [
            if not immune-to? myself [ ; Disease fought off immediately.
              set my-diseases (turtle-set my-diseases myself)
              ; Note: infected with perfect copy of disease. No mutation occurs during transmission.
            ]
          ]
        ]
      ]
    ]
  ]
end

to-report immune-to? [given-disease]
  report not (false = position ([code] of given-disease) (array:to-list immune-system))
end

to fight-my-diseases
  let beaten-diseases []
  let cur-in-pos 0
  let cur-out-pos 0
  let num-matches 0
  let max-num-matches 0
  let mismatch-pos 0
  let num-mismatches 0
  
  ask my-diseases [
    set cur-in-pos 0
    set cur-out-pos 0
    
    while [(cur-out-pos + length code) < immune-system-length] [
      set num-matches 0
      set cur-in-pos cur-out-pos
      foreach code [
        set num-matches num-matches + ifelse-value (? = [array:item immune-system cur-in-pos] of myself) [1] [0]
        set cur-in-pos 1 + cur-in-pos
      ]
      array:set immune-sys-matches cur-out-pos num-matches
      set cur-out-pos 1 + cur-out-pos
    ]
    repeat ((length code) - 1) [
      array:set immune-sys-matches cur-out-pos 0
      set cur-out-pos 1 + cur-out-pos
    ]
    
    set max-num-matches max array:to-list immune-sys-matches
    ifelse (length code) = max-num-matches [
      set beaten-diseases fput self beaten-diseases
    ]
    [
      set mismatch-pos position max-num-matches (array:to-list immune-sys-matches) ; If substrings tie, this will take the first.
      set cur-out-pos first-mismatch-position myself mismatch-pos
      ask myself [
        array:set immune-system cur-out-pos (1 - array:item immune-system cur-out-pos)
      ]
      
    ]
  ]
  if not empty? beaten-diseases [
    set my-diseases my-diseases with [not member? self beaten-diseases]
  ]
end

to-report first-mismatch-position [given-agent start-pos]
  let cur-pos start-pos
  foreach code [
    if ? != [array:item immune-system cur-pos] of given-agent [report cur-pos]
    set cur-pos 1 + cur-pos
  ]
  report false ; No mismatch found!
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Plotting

to initialize-plots
  set-current-plot "Groups"
  let cur-group 0
  set-plot-pen-color array:item group-colors cur-group
  repeat (number-of-groups - 1) [
    set cur-group 1 + cur-group
    create-temporary-plot-pen (word cur-group)
    set-plot-pen-color array:item group-colors cur-group
  ]
  
end

to do-plots
  set-current-plot "Population"
  plotxy ticks (count agents)
  
  if any? agents [
    set-current-plot "Attributes"
    set-current-plot-pen "Vision"
    plotxy ticks (mean [vision] of agents)
    set-current-plot-pen "Metabolic Rate"
    plotxy ticks (mean [metabolic-rate] of agents)
    if spice? [
      set-current-plot-pen "Spice Metabolic Rate"
      plotxy ticks (mean [spice-metabolic-rate] of agents)
      set-current-plot-pen "Foresight"
      plotxy ticks (mean [foresight] of agents)
    ]
    
    set-current-plot "Wealth"
    set-plot-x-range 0 (10 + precision (max [sugar-stock] of agents) -1)
    set-plot-y-range 0 1
    set-histogram-num-bars 10
    histogram [round sugar-stock] of agents
    
    if spice? [
      set-current-plot "Welfare"
      set-plot-x-range 0 (10 + precision (max [welfare 0 0] of agents) -1)
      set-plot-y-range 0 1
      set-histogram-num-bars 10
      histogram [round (welfare 0 0)] of agents
      
      set-current-plot "Agents' Wealth"
      clear-plot
      ask agents [plotxy sugar-stock spice-stock]
            
      if not empty? trade-prices [
        set-current-plot "Trade Prices"
        set-current-plot-pen "Geomean"
        plotxy ticks (geomean trade-prices)
;        set-current-plot-pen "Max"
;        plotxy ticks (max trade-prices)
;        set-current-plot-pen "Min"
;        plotxy ticks (min trade-prices)
        
        set-current-plot "Trade Volume"
        plotxy ticks (length trade-prices)
        if 1 < length trade-prices [
          set-current-plot "Stdev Log Prices"
          plotxy ticks (standard-deviation (map [log ? 10] trade-prices))
        ]
      ]
    ]
    
    set-current-plot "Endowment"
    set-plot-x-range 0 (1 + max [endowment] of agents)
    set-plot-y-range 0 1
    set-histogram-num-bars 10
    histogram [endowment] of agents
    
    set-current-plot "Age"
    set-plot-x-range 0 (1 + max-death-age)
    set-plot-y-range 0 1
    set-histogram-num-bars 10
    histogram [age] of agents
    
    set-current-plot "Children"
    set-plot-x-range 0 (1 + max [length children] of agents)
    set-plot-y-range 0 1
    set-plot-pen-interval 1
    histogram [length children] of agents
    
    set-current-plot "Population Changes"
    set-current-plot-pen "Births"
    plotxy ticks num-babies
 ;   set-current-plot-pen "All Deaths"
 ;   plotxy ticks length ages-at-death
    set-current-plot-pen "Starved"
    plotxy ticks num-starved
    set-current-plot-pen "Killed"
    plotxy ticks num-killed
    set-current-plot-pen "Old Age"
    plotxy ticks ((length ages-at-death) - num-starved - num-killed)
    
    if 0 < length ages-at-death [
      set-current-plot "Age at Death"
      set-plot-x-range 0 (1 + max-death-age)
      set-plot-y-range 0 1
      set-histogram-num-bars 10
      histogram ages-at-death
    ]
    
    if culture? [
      let tag-freqs n-values number-of-tags [0]
      let group-sizes array:from-list n-values number-of-groups [0]
      let mean-tags 0
      let min-tags number-of-tags
      let max-tags 0
      let cur-tags 0
      ask agents [
        array:set group-sizes group 1 + (array:item group-sizes group)
        set tag-freqs (map [?1 + ?2] tag-freqs (array:to-list tags))
        set cur-tags sum (array:to-list tags)
        set mean-tags mean-tags + cur-tags
        if max-tags < cur-tags [set max-tags cur-tags]
        if min-tags > cur-tags [set min-tags cur-tags]
      ]
      set-current-plot "Tags"
      clear-plot
      let cur-item 0
      repeat number-of-tags [
        plotxy cur-item (item cur-item tag-freqs)
        set cur-item 1 + cur-item
      ]
      set-current-plot "Tags Evolution"
      set-current-plot-pen "Mean"
      plotxy ticks (mean-tags / (count agents))
      set-current-plot-pen "Min"
      plotxy ticks min-tags
      set-current-plot-pen "Max"
      plotxy ticks max-tags
      
      set-current-plot "Groups"
      set cur-item 0
      repeat number-of-groups [
        set-current-plot-pen (word cur-item)
        plotxy ticks (array:item group-sizes cur-item)
        set cur-item 1 + cur-item
      ]
    ]
     
    calc-gini-and-lorenz
  ]

  if disease? [
    set-current-plot "Diseases"
;    set-plot-x-range 0 (1 + max-disease-id)
;    set-plot-pen-interval 1
;    if any? agents with [any? my-diseases] [
;      histogram (reduce [sentence ?1 ?2] ([([disease-id] of my-diseases)] of agents))
;    ]
    
    let num-0-diseases 0
    let num-1-diseases 0
    let num-2-diseases 0
    set disease-freq array:from-list n-values max-disease-id [0]
    ask agents [
      ifelse 0 = count my-diseases [
        set num-0-diseases 1 + num-0-diseases
      ]
      [
        ifelse 1 < count my-diseases [
          set num-2-diseases 1 + num-2-diseases
        ]
        [
          set num-1-diseases 1 + num-1-diseases
        ]
        ask my-diseases [
          array:set disease-freq disease-id (1 + array:item disease-freq disease-id)
        ]
      ]
    ]
    
    clear-plot
    let cur-disease-id 0
    foreach array:to-list disease-freq [
      plotxy cur-disease-id ?
      set cur-disease-id 1 + cur-disease-id
    ]
    
    set-current-plot "Infected"
    set-current-plot-pen "# Diseases > 1"
    plotxy ticks num-2-diseases ;/ (count agents))
    set-current-plot-pen "# Diseases = 1"
    plotxy ticks num-1-diseases ;/ (count agents))
    set-current-plot-pen "Clear"
    plotxy ticks num-0-diseases ;/ (count agents))
    
  ]

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to calc-gini-and-lorenz
  let cur-pos 0
  let cur-total 0.0
  let final-total 0.0 + sum [sugar-stock] of agents
  let gini-total 0.0
  
  set-current-plot "Lorenz"
  clear-plot
  
  set-current-plot-pen "y = x"
  plotxy 0 0
  plotxy 100 100
  
  set-current-plot-pen "Lorenz"
  foreach sort-on [sugar-stock] agents [
    set cur-pos cur-pos + 1
    set cur-total cur-total + [sugar-stock] of ?
;    set gini-total gini-total + (cur-pos / (count agents)) - (cur-total / final-total)
    set gini-total gini-total + (cur-pos * ([sugar-stock] of ?))
    if final-total > 0 [plotxy (100 * cur-pos / (count agents)) (100 * cur-total / final-total)]
  ]
  set gini ifelse-value (final-total <= 0) [0] [((2 * gini-total / final-total) - (1 + count agents) ) / (count agents)]
  
  set-current-plot "Gini Coefficient"
  plotxy ticks gini
  
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Predefined scenarios

to setup-scenario-given-params [
    sim-params
    visuals-params
    agent-params
    environment-params
    seasons-params
    pollution-params
    culture-params
    lending-params
    disease-params
]
  set scape-type "Classic: 2 hills of Sugar & 2 hills of Spice"
  let params []
  
  if not empty? sim-params [
    set params array:from-list sim-params
    set initial-population array:item params 0
    set scape-type array:item params 1
    set max-ticks array:item params 2
    set spice? array:item params 3
    set trade? array:item params 4
    set seasons? array:item params 5
    set pollution? array:item params 6
    set culture? array:item params 7
    set combat? array:item params 8
    set lending? array:item params 9
    set disease? array:item params 10
    set rng-seed array:item params 11
  ]
  
  if not empty? visuals-params [
    set params array:from-list visuals-params
    set patch-color-scheme array:item params 0
    set agent-color-scheme array:item params 1
    set print-trades? array:item params 2
    set link-parentage? array:item params 3
    set hide-loan-links? array:item params 4
  ]
  
  if not empty? agent-params [
    set params array:from-list agent-params
    set mean-vision array:item params 0
    set mean-metabolic-rate array:item params 1
    set initial-location array:item params 2
    set initial-age-0? array:item params 3
    set min-death-age array:item params 4
    set max-death-age array:item params 5
    set min-endowment array:item params 6
    set max-endowment array:item params 7
    
    set reproduction array:item params 8
    set inheritance? array:item params 9
    
    set min-fertility-start array:item params 10
    set max-fertility-start array:item params 11
    set min-fertility-end-male array:item params 12
    set max-fertility-end-male array:item params 13
    set min-fertility-end-female array:item params 14
    set max-fertility-end-female array:item params 15

    set mean-foresight array:item params 16
    
  ]
  
  if not empty? environment-params [
    set params array:from-list environment-params
    set growback-method array:item params 0
    set sugar-growback-rate array:item params 1
    set spice-growback-rate array:item params 2
  ]
  
  if not empty? seasons-params [
    set params array:from-list seasons-params
    set off-season-inter-growback-time array:item params 0
    set season-length array:item params 1
  ]

  if not empty? pollution-params [
    set params array:from-list pollution-params
    set production-pollution array:item params 0
    set consumption-pollution array:item params 1
    set pollution-diffusion-period array:item params 2
    set pollution-on-at-time array:item params 3
    set pollution-diffusion-on-at array:item params 4
  ]

  if not empty? culture-params [
    set params array:from-list culture-params
    set number-of-tags array:item params 0
    set number-of-groups array:item params 1
  ]

  if not empty? lending-params [
    set params array:from-list lending-params
    set loan-period array:item params 0
    set interest-rate array:item params 1
  ]

  if not empty? disease-params [
    set params array:from-list disease-params
    set number-of-diseases array:item params 0
    set agents-initial-number-of-diseases array:item params 1
    set immune-system-length array:item params 2
    set mean-disease-code-length array:item params 3
    set mean-disease-effect array:item params 4
  ]

end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

to setup-scenario
  run (word "setup-scenario-" (substring scenario 0 (position ":" scenario)))
  setup
end

to setup-scenario-0
  ; 0: No change to parameters
end

to setup-scenario-1
  ; Movement (Ch. 2, p.30-1)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 500 false false false false false false false false 0]
  ; visuals-params
  ["Sugar" "Red" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "None" false 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] [] [] []
end

to setup-scenario-2
  ; Replacement (Ch. 2, p.33)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [250 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 false false false false false false false false 0]
  ; visuals-params
  ["Sugar" "Red" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "On death" false 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] [] [] []
end

to setup-scenario-3
  ; Migration (Ch. 2, p.42)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 false false false false false false false false 0]
  ; visuals-params
  ["Sugar" "Vision & Metabolic Rate" false false true]
  ; agent-params
  [5.5 2.5 "Corner" true 60 100 5 25 "None" false 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] [] [] []
end

to setup-scenario-4
  ; Seasonal Migration (Ch. 2, p.44)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 false false true false false false false false 0]
  ; visuals-params
  ["Sugar" "Birth-Place" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "On death" false 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  ; seasons-params
  [8 50]
  [] [] [] []
end

to setup-scenario-5
  ; Pollution (Ch. 2, p.47-8)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 false false false true false false false false 0]
  ; visuals-params
  ["Pollution" "Red" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "None" false 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  []
  ; pollution-params
  [1 1 1 50 100]
  [] [] []
end

to setup-scenario-6
  ; Sexual Reproduction (Ch. 3, p.58-67)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 2500 false false false false false false false false 0]
  ; visuals-params
  ["Sugar" "Fertility" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 10 40 "Sexual" false 12 15 40 50 30 40 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] [] [] []
end

to setup-scenario-7
  ; Inheritance (Ch. 3, p.67)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 false false false false false false false false 0]
  ; visuals-params
  ["Sugar" "Endowment" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "Sexual" true 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] [] [] []
end

to setup-scenario-8
  ; Cultural Transmission & Groups (Ch. 3, p.73)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 false false false false true false false false 0]
  ; visuals-params
  ["Sugar" "Cultural Group" false false true]
  ; agent-params
  [3.5 2.5 "2 Group Corners" true 60 100 5 25 "Sexual" true 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] 
  ; culture-params
  [11 2]
  [] []
end

to setup-scenario-9
  ; Combat (Ch. 3, p.82-3)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 false false false false true true false false 0]
  ; visuals-params
  ["Sugar" "Cultural Group" false false true]
  ; agent-params
  [3.5 2.5 "2 Group Corners" true 60 100 5 25 "Sexual" true 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] 
  ; culture-params
  [11 2]
  [] []
end

to setup-scenario-10
  ; Sugar & Spice (Ch. 4)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 true false false false false false false false 0]
  ; visuals-params
  ["Sugar & Spice" "Welfare" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "None" false 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] [] [] []
end

to setup-scenario-11
  ; Trade (Ch. 4, p. 105-9)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 true true false false false false false false 0]
  ; visuals-params
  ["Sugar & Spice" "Welfare" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "None" false 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] [] [] []
end

to setup-scenario-12
  ; Foresight (Ch. 4, p. 129)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 true true false false false false false false 0]
  ; visuals-params
  ["Sugar & Spice" "Foresight" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "Sexual" false 12 15 50 60 40 50 3.5]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] [] [] []
end

to setup-scenario-13
  ; Lending (Ch. 4, p. 131-3)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 false false false false false false true false 0]
  ; visuals-params
  ["Sugar" "Lending" false false false]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "Sexual" false 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] []
  ; lending-params
  [10 10]
  []
end

to setup-scenario-14
  ; Disease & Immunity (Ch. 5, p. 144-5)
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 false false false false false false false true 0]
  ; visuals-params
  ["Sugar" "Diseases" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 5 25 "Sexual" true 12 15 50 60 40 50 0]
  ; environment-params
  ["Fixed-Rate" 1 1]
  [] [] [] []
  ; disease-params
  [25 10 50 5.5 1]
  
;  user-message "Code for Diseases has not yet been finished!"
end

to setup-scenario-15
  ; All processes together
  setup-scenario-given-params
  ; sim-params
  ; Rules: spice? trade? seasons? pollution? culture? combat? lending? disease? 
  [400 "Classic: 2 hills of Sugar & 2 hills of Spice" 1000 true true true true true true true true 0]
  ; visuals-params
  ["Sugar & Spice" "Cultural Group" false false true]
  ; agent-params
  [3.5 2.5 "Anywhere" true 60 100 25 50 "Sexual" true 12 15 50 60 40 50 3.5]
  ; environment-params
  ["Fixed-Rate" 1 1]
  ; seasons-params
  [8 50]
  ; pollution-params
  [1 1 1 50 100]
  ; culture-params
  [11 2]
  ; lending-params
  [10 10]
  ; disease-params
  [25 10 50 5.5 1]
end

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@#$#@#$#@
GRAPHICS-WINDOW
538
10
948
441
-1
-1
8.0
1
10
1
1
1
0
0
0
1
0
49
0
49
0
0
1
ticks
30.0

BUTTON
316
141
380
174
Setup
Setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
383
141
446
174
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
448
141
511
174
Step
go
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

CHOOSER
10
667
172
712
Growback-Method
Growback-Method
"Instantly-Max" "Fixed-Rate" "Poisson"
1

PLOT
960
11
1160
161
Population
Time (ticks)
Count Agents
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""

SLIDER
9
209
181
242
Mean-Vision
Mean-Vision
1
20
3.5
0.5
1
patches
HORIZONTAL

SLIDER
9
245
209
278
Mean-Metabolic-Rate
Mean-Metabolic-Rate
1
4.5
2.5
0.5
1
units / tick
HORIZONTAL

MONITOR
1166
166
1255
211
NIL
Count Agents
17
1
11

PLOT
1165
10
1505
160
Attributes
Time (ticks)
Mean Value
0.0
1.0
0.0
1.0
true
true
"" ""
PENS
"Vision" 1.0 0 -14730904 true "" ""
"Metabolic Rate" 1.0 0 -2674135 true "" ""
"Spice Metabolic Rate" 1.0 0 -13840069 true "" ""
"Foresight" 1.0 0 -8630108 true "" ""

PLOT
960
166
1160
316
Wealth
Sugar Stock
Count Agents
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

TEXTBOX
7
10
171
46
SugarscapeCW
24
0.0
1

PLOT
961
335
1161
485
Lorenz
% Population
% Wealth
0.0
100.0
0.0
100.0
true
false
"" ""
PENS
"y = x" 1.0 0 -4539718 true "" ""
"Lorenz" 1.0 0 -10899396 true "" ""

MONITOR
1368
335
1435
380
NIL
Gini
3
1
11

PLOT
1165
335
1365
485
Gini Coefficient
Time (ticks)
Gini
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""

TEXTBOX
11
185
161
203
Agents' initial attributes:
13
0.0
1

TEXTBOX
12
643
162
661
Environment:
13
0.0
1

MONITOR
249
197
321
242
Max-Vision
(2 * mean-vision) - 1
1
1
11

MONITOR
213
244
321
289
Max-Metabolic-Rate
(2 * mean-metabolic-rate) - 1
1
1
11

INPUTBOX
10
418
165
478
Min-Death-Age
60
1
0
Number

INPUTBOX
168
418
323
478
Max-Death-Age
100
1
0
Number

INPUTBOX
9
283
164
343
Min-Endowment
5
1
0
Number

INPUTBOX
166
283
321
343
Max-Endowment
25
1
0
Number

SLIDER
7
104
179
137
Initial-Population
Initial-Population
0
1000
400
10
1
NIL
HORIZONTAL

CHOOSER
328
206
468
251
Initial-Location
Initial-Location
"Anywhere" "Corner" "2 Group Corners" "4 Group Corners"
0

CHOOSER
8
54
318
99
Scenario
Scenario
"1: Movement (Ch. 2, p.30-1)" "2: Replacement (Ch. 2, p.33)" "3: Migration (Ch. 2, p.42)" "4: Seasonal Migration (Ch. 2, p.44)" "5: Pollution (Ch. 2, p.47-8)" "6: Sexual Reproduction (Ch. 3, p.58-67)" "7: Inheritance (Ch. 3, p.67)" "8: Cultural Transmission & Groups (Ch. 3, p.73)" "9: Combat (Ch. 3, p.82-3)" "10: Sugar & Spice (Ch. 4)" "11: Trade (Ch. 4, p. 105-9)" "12: Foresight (Ch. 4, p. 129)" "13: Lending (Ch. 4, p. 131-3)" "14: Disease & Immunity (Ch. 5, p. 144-5)" "15: All processes together" "0: No change to parameters"
0

SLIDER
123
722
379
755
Off-Season-Inter-Growback-Time
Off-Season-Inter-Growback-Time
1
20
8
1
1
ticks
HORIZONTAL

SLIDER
176
667
348
700
Sugar-Growback-Rate
Sugar-Growback-Rate
0
10
1
1
1
NIL
HORIZONTAL

SLIDER
123
757
295
790
Season-Length
Season-Length
1
100
50
1
1
ticks
HORIZONTAL

BUTTON
323
54
437
87
Setup Scenario
Setup-Scenario
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

MONITOR
386
723
460
768
Summer in:
ifelse-value (0 = (int (ticks / season-length)) mod 2) [\"North\"] [\"South\"]
17
1
11

SWITCH
10
794
120
827
Pollution?
Pollution?
1
1
-1000

SLIDER
123
829
331
862
Consumption-Pollution
Consumption-Pollution
0
5
1
1
1
sugar units
HORIZONTAL

SLIDER
123
794
331
827
Production-Pollution
Production-Pollution
0
5
1
1
1
sugar units
HORIZONTAL

SLIDER
123
864
330
897
Pollution-Diffusion-Period
Pollution-Diffusion-Period
1
100
1
1
1
ticks
HORIZONTAL

CHOOSER
540
478
679
523
Patch-Color-Scheme
Patch-Color-Scheme
"Sugar" "Sugar & Spice" "Spice" "Pollution" "Sugar / Pollution"
0

INPUTBOX
333
794
488
854
Pollution-On-At-Time
50
1
0
Number

INPUTBOX
333
856
488
916
Pollution-Diffusion-On-At
100
1
0
Number

BUTTON
682
480
762
513
Recolour
ask patches [ recolour-patch ]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
10
527
211
560
Min-Fertility-Start
Min-Fertility-Start
0
100
12
1
1
ticks
HORIZONTAL

SLIDER
213
527
415
560
Max-Fertility-Start
Max-Fertility-Start
0
100
15
1
1
ticks
HORIZONTAL

SLIDER
10
563
211
596
Min-Fertility-End-Male
Min-Fertility-End-Male
0
100
50
1
1
ticks
HORIZONTAL

SLIDER
213
563
415
596
Max-Fertility-End-Male
Max-Fertility-End-Male
0
100
60
1
1
ticks
HORIZONTAL

SLIDER
10
598
211
631
Min-Fertility-End-Female
Min-Fertility-End-Female
0
100
40
1
1
ticks
HORIZONTAL

SLIDER
213
598
415
631
Max-Fertility-End-Female
Max-Fertility-End-Female
0
100
50
1
1
ticks
HORIZONTAL

CHOOSER
10
480
148
525
Reproduction
Reproduction
"None" "On death" "Sexual" "Sexual(CW)"
0

CHOOSER
540
525
717
570
Agent-Color-Scheme
Agent-Color-Scheme
"Red" "Birth-Place" "Age" "Fertility" "Sex" "Vision" "Metabolic Rate" "Vision & Metabolic Rate" "Sugar-Stock" "Spice-Stock" "Endowment" "Foresight" "Children" "Cultural Group" "Welfare" "Lending" "Diseases"
0

BUTTON
721
525
791
558
Recolour
ask agents [ recolour-agent ]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
1260
167
1460
317
Age
ticks
Count Agents
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

MONITOR
1229
535
1321
580
Recent Dead
length ages-at-death
17
1
11

PLOT
1409
484
1609
634
Age at Death
ticks
Count Agents
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

BUTTON
1369
389
1432
422
Go
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

PLOT
961
488
1227
638
Population Changes
Time (ticks)
Count Agents
0.0
1.0
0.0
1.0
true
true
"" ""
PENS
"Births" 1.0 0 -11085214 true "" ""
"Starved" 1.0 0 -2674135 true "" ""
"Killed" 1.0 0 -7500403 true "" ""
"Old Age" 1.0 0 -10141563 true "" ""

MONITOR
1229
488
1321
533
Recent Births
num-babies
17
1
11

PLOT
1463
168
1663
318
Endowment
Units Sugar
Count Agents
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

SWITCH
11
722
119
755
Seasons?
Seasons?
1
1
-1000

PLOT
1612
484
1812
634
Children
Count Children
Count Agents
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

SWITCH
151
480
276
513
Inheritance?
Inheritance?
1
1
-1000

SWITCH
795
478
939
511
Link-Parentage?
Link-Parentage?
1
1
-1000

MONITOR
796
516
909
561
NIL
Count Parentages
17
1
11

SWITCH
278
480
407
513
Initial-Age-0?
Initial-Age-0?
0
1
-1000

SWITCH
547
666
650
699
Culture?
Culture?
1
1
-1000

SLIDER
654
666
826
699
Number-Of-Tags
Number-Of-Tags
1
64
11
1
1
NIL
HORIZONTAL

SLIDER
654
703
826
736
Number-Of-Groups
Number-Of-Groups
1
8
2
1
1
NIL
HORIZONTAL

PLOT
961
641
1228
809
Groups
Time (ticks)
Count Members
0.0
1.0
0.0
1.0
true
true
"" ""
PENS
"0" 1.0 0 -16777216 true "" ""

PLOT
1231
640
1431
809
Tags
Tag
Count Agents
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

SWITCH
546
703
650
736
Combat?
Combat?
1
1
-1000

SWITCH
186
104
289
137
Spice?
Spice?
1
1
-1000

SLIDER
350
667
522
700
Spice-Growback-Rate
Spice-Growback-Rate
0
10
1
1
1
NIL
HORIZONTAL

TEXTBOX
551
641
740
659
Cultural, groups & combat:
13
0.0
1

TEXTBOX
14
400
164
418
Age & Reproduction:
13
0.0
1

TEXTBOX
187
10
490
41
A replication of the classic model in Epstein & Axtell (1996).\nThis program (C) Christopher Watts, 2015.
11
0.0
1

SWITCH
186
140
289
173
Trade?
Trade?
1
1
-1000

MONITOR
1324
582
1406
627
Recent Starved
num-starved
17
1
11

MONITOR
1323
535
1406
580
Recent Killed
num-killed
17
1
11

SWITCH
541
587
670
620
Print-Trades?
Print-Trades?
1
1
-1000

INPUTBOX
542
1103
697
1163
RNG-Seed
0
1
0
Number

BUTTON
702
1114
816
1147
Print Last Seed
print \"\"\nprint \"Seed number for most recent simulation run:\"\nprint \"(To repeat run, copy-paste this number to RNG-Seed.)\"\nprint last-rng-seed\n
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

INPUTBOX
450
60
520
120
Max-ticks
500
1
0
Number

PLOT
961
812
1228
1022
Trade Prices
Time (ticks)
Average Trade Price
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"Geomean" 1.0 2 -16777216 true "" ""
"Max" 1.0 2 -955883 true "" ""
"Min" 1.0 2 -13840069 true "" ""

PLOT
1231
812
1512
1021
Trade Volume
Time (ticks)
Count Trades
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 1 -7500403 true "" ""

PLOT
1515
813
1757
1023
Stdev Log Prices
Time (ticks)
NIL
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 2 -16777216 true "" ""

PLOT
1439
332
1639
482
Welfare
Welfare
Count Agents
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

SLIDER
9
347
181
380
Mean-Foresight
Mean-Foresight
0
10
0
.5
1
ticks
HORIZONTAL

MONITOR
186
347
279
392
Max-Foresight
(2 * mean-foresight)
17
1
11

SWITCH
796
565
944
598
Hide-Loan-Links?
Hide-Loan-Links?
0
1
-1000

TEXTBOX
546
750
696
768
Loans:
13
0.0
1

SLIDER
650
775
822
808
Loan-Period
Loan-Period
1
100
10
1
1
ticks
HORIZONTAL

SLIDER
650
810
879
843
Interest-Rate
Interest-Rate
0
100
10
1
1
% per loan-period
HORIZONTAL

SWITCH
544
775
648
808
Lending?
Lending?
1
1
-1000

TEXTBOX
11
934
161
952
Diseases:
13
0.0
1

SLIDER
123
955
295
988
Number-Of-Diseases
Number-Of-Diseases
0
100
25
1
1
NIL
HORIZONTAL

SLIDER
8
991
250
1024
Agents-Initial-Number-Of-Diseases
Agents-Initial-Number-Of-Diseases
0
20
10
1
1
NIL
HORIZONTAL

SWITCH
9
955
119
988
Disease?
Disease?
1
1
-1000

SLIDER
9
1028
235
1061
Mean-Disease-Code-Length
Mean-Disease-Code-Length
1
20
5.5
.5
1
bits
HORIZONTAL

MONITOR
238
1029
397
1074
Max-Disease-Code-Length
(2 * mean-disease-code-length) - 1
17
1
11

SLIDER
253
991
464
1024
Immune-System-Length
Immune-System-Length
1
100
50
1
1
bits
HORIZONTAL

SLIDER
8
1076
237
1109
Mean-Disease-Effect
Mean-Disease-Effect
1
10
1
.5
1
units/tick
HORIZONTAL

MONITOR
243
1077
371
1122
Max-Disease-Effect
(2 * Mean-Disease-Effect) - 1
17
1
11

PLOT
962
1025
1162
1175
Diseases
Disease Who
Count Infected
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 1 -16777216 true "" ""

PLOT
1164
1025
1423
1175
Infected
Time (ticks)
Count Agents
0.0
1.0
0.0
1.0
true
true
"" ""
PENS
"# Diseases > 1" 1.0 0 -8053223 true "" ""
"# Diseases = 1" 1.0 0 -1604481 true "" ""
"Clear" 1.0 0 -13345367 true "" ""

TEXTBOX
542
1077
715
1095
Random Number Generation:
13
0.0
1

TEXTBOX
538
453
688
471
Outputs & Visuals:
13
0.0
1

SLIDER
543
984
715
1017
Grid-Width
Grid-Width
20
100
50
5
1
Patches
HORIZONTAL

SLIDER
543
1020
715
1053
Max-Resources
Max-Resources
1
8
4
1
1
Units
HORIZONTAL

TEXTBOX
544
890
694
908
Alternative Scapes:
13
0.0
1

CHOOSER
543
913
869
958
Scape-Type
Scape-Type
"Classic: 2 hills of Sugar & 2 hills of Spice" "Flat Grid: Units of Sugar & Spice = Max-Resources"
0

MONITOR
1509
11
1632
56
# Agents / # Patches
(count agents) / (count patches)
3
1
11

MONITOR
719
984
849
1029
Initial Grid Pop. Density
initial-population / (grid-width * grid-width)
3
1
11

TEXTBOX
545
967
695
985
Grid Parameters:
11
0.0
1

PLOT
1433
640
1710
809
Tags Evolution
Time (ticks)
Count of 1s
0.0
1.0
0.0
1.0
true
true
"" ""
PENS
"Max" 1.0 0 -2674135 true "" ""
"Mean" 1.0 0 -13840069 true "" ""
"Min" 1.0 0 -13345367 true "" ""

PLOT
1667
286
1867
482
Agents' Wealth
Sugar
Spice
0.0
1.0
0.0
1.0
true
false
"" ""
PENS
"default" 1.0 2 -16777216 true "" ""

BUTTON
829
1056
958
1089
Print Disease Info
print \"\"\nprint \"Diseases (Effect : # Infected : Code):\"\nforeach sort diseases [\nask ? [\nshow (word metabolic-effect \" : \" (array:item disease-freq disease-id) \" : \" code)\n]\n]
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

@#$#@#$#@
# CW's SUGARSCAPE

## WHAT IS IT?

This program attempts to replicate the classic model, Sugarscape, as described in Epstein & Axtell (1996). Most of the functions are provided, and qualitatively similar (if not numerically indistinguishable) results can be obtained from experiments in most cases. Many people have tried to replicate some or all of Sugarscape. This is my attempt.

This program (C) Christopher Watts, 2015.

## HOW IT WORKS

The functionality and thinking behind the model's design can be found in Epstein & Axtell (1996), and will not be reproduced here. For further thoughts on the design, see Watts (2015).

There is one breed of turtles, called "agents". The sugarscape itself is made up of patches. Agents move around the landscape, seeking patches with spare resource, which the agents need to replenish their stocks. In the base model, there is one resource, called "sugar". Later scenarios can include a second resource, called "spice". Agents whose stocks run out will die.

New agents can enter the model through some form of reproduction: either initiated whenever another agent dies (so that the population remains constant), or the result of sexual reproduction between adjacent male and female agents of fertile age. Children from sexual reproduction inherit attributes from their parents, and may optionally also inherit a share of their wealth when the parents die.

## HOW TO USE IT

Choose parameter values and menu options. Then click "Setup" and "Go". 

You can also click "Setup Scenario" to jump to preset scenarios, roughly corresponding to those described in the book. These are:

  1. Movement (Chapter 2, p.30-1)
  2. Replacement (Ch. 2, p.33)
  3. Migration (Ch. 2, p.42)
  4. Seasonal Migration (Ch. 2, p.44)
  5. Pollution (Ch. 2, p.47-8)
  6. Sexual Reproduction (Ch. 3, p.58-67)
  7. Inheritance (Ch. 3, p.67)
  8. Cultural Transmission & Groups (Ch. 3, p.73)
  9. Combat (Ch. 3, p.82-3)
  10. Sugar & Spice (Ch. 4)
  11. Trade (Ch. 4, p. 105-9)
  12. Foresight (Ch. 4, p. 129)
  13. Lending (Ch. 4, p. 131-3)
  14. Disease & Immunity (Ch. 5, p. 144-5)
  15. All processes together

## THINGS TO NOTICE

Agents will migrate over the sugarscape, and their numbers may rise and fall. The distribution of wealth (i.e. sugar) is plotted, along with the calculation of a Lorenz curve and Gini coefficient to show any inequality in this distribution. Depending on the parameter settings, log-normal distributions or scale-free (Pareto) distributions may be seen.

## THINGS TO TRY

A key challenge is to obtain a self-sustaining population. After this, see how large a population the sugarscape can carry.

Which factors have the most influence over the sustainability of the society and the carrying capacity of the sugarscape?

Try the alternative scape: a square grid of patches uniform in their sugar and spice resources ("Max-Resources"). What is the relation between grid size, max-resources, mean metabolic rates and carrying capacity?

## EXTENDING THE MODEL

Epstein & Axtell's attempt at modelling an artificial socio-economic system using only the simplest assumptions included several important assumptions that others might question. 

Individualism: Agents act as individuals responsible for their own survival. Evidence from archaeology and anthropology suggests that such individualism is not the most realistic assumption for modelling ancient communities. Rather individuals produced items for their small community, and consumed food and other items from this communal store. Trade was initially between communities, not between individual members. How might we represent this alternative theory in Sugarscape? Might comparing models tell us something about the relative success of individualist and communalist societies?

Homes: The agents have no attachment to any particular patch or area of patches. Why do real human agents have homes? What advantages stem from this? What causes humans to migrate?

Family: Family relations are not acted on. Males and females gather sugar and engage in combat at the same rate as each other. They do not collect resources for partners, parents or children.

Age determines fertility and death from old age but nothing else. So 1-year-old children engage in migration, consumption and combat at the same rate as mature adults and the elderly.

Groups: Cultural groups do not influence the distribution of resources, other than through combat. Work on tags models by Axelrod and colleagues demonstrates how homophily can form the basis of cooperative norms.

## RELATED MODELS

There have been many attempts to replicate some or all of the Sugarscape model. Iain Weaver's (2009) Netlogo version seems particularly polished and extensive, and it achieves a similar range of features to the current program. See:
http://ccl.northwestern.edu/netlogo/models/community/Sugarscape
http://www2.le.ac.uk/departments/interdisciplinary-science/research/the-sugarscape.

Sugarscape can be seen as the fore-runner to various models of land-use and sustainable resource use, artificial economies and market trading. Along with Axelrod (1997), and some of the work at the Santa Fe Institute, Epstein & Axtell (1996) were among the first major contributions to agent-based simulation modelling of social phenomena.

Axtell et al. (1996) attempted to adapt the Sugarscape model to replicate Axelrod's model of cultural influence.

## CREDITS AND REFERENCES

Epstein, J. M., & Axtell, R. (1996). Growing artificial societies : social science from the bottom up. Washington, D.C. ; London: Brookings Institution Press ; MIT Press.

Axelrod, R. M. (1997). The complexity of cooperation : agent-based models of competition and collaboration. Princeton, N.J. ; Chichester: Princeton University Press.

Axtell, R., Axelrod, R., Epstein, J., & Cohen, M. (1996). Aligning simulation models: A case study and results. Computational & Mathematical Organization Theory, 1(2), 123-141. doi: 10.1007/BF01299065

Watts, C. (2015) Sugarscape re-evaluated: an appreciation of an early classic in social simulation, with proposals for the future development of Artificial Economics. Paper presented at the ESSA Social Simulation Conference 2015, Gronningen, NL.

Weaver, Iain, (2009) Sugarscape. NetLogo User Community Models: Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.2.0
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
<experiments>
  <experiment name="experiment" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <timeLimit steps="500"/>
    <metric>timer</metric>
    <metric>count agents</metric>
    <metric>gini</metric>
    <metric>mean [vision] of agents</metric>
    <metric>mean [metabolic-rate] of agents</metric>
    <metric>mean [sugar-stock] of agents</metric>
    <metric>min [vision] of agents</metric>
    <metric>min [metabolic-rate] of agents</metric>
    <metric>min [sugar-stock] of agents</metric>
    <metric>max [vision] of agents</metric>
    <metric>max [metabolic-rate] of agents</metric>
    <metric>max [sugar-stock] of agents</metric>
    <metric>standard-deviation [vision] of agents</metric>
    <metric>standard-deviation [metabolic-rate] of agents</metric>
    <metric>standard-deviation [sugar-stock] of agents</metric>
    <enumeratedValueSet variable="Initial-Population">
      <value value="500"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Replace-Dead-Agents?">
      <value value="false"/>
    </enumeratedValueSet>
    <steppedValueSet variable="Mean-Vision" first="1" step="1" last="10"/>
    <enumeratedValueSet variable="Mean-Metabolic-Rate">
      <value value="1"/>
      <value value="2"/>
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Sugar-Growback-Method">
      <value value="&quot;Fixed-Rate&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Sugar-Growback-Rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Endowment">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Endowment">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Death-Age">
      <value value="60"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Death-Age">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Vision">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Metabolic-Rate">
      <value value="1"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment-Trade" repetitions="40" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>timer</metric>
    <metric>count agents</metric>
    <metric>gini</metric>
    <metric>mean [vision] of agents</metric>
    <metric>mean [metabolic-rate] of agents</metric>
    <metric>mean [sugar-stock] of agents</metric>
    <metric>mean [spice-stock] of agents</metric>
    <metric>median [sugar-stock] of agents</metric>
    <metric>median [spice-stock] of agents</metric>
    <metric>min [vision] of agents</metric>
    <metric>min [metabolic-rate] of agents</metric>
    <metric>min [sugar-stock] of agents</metric>
    <metric>min [spice-stock] of agents</metric>
    <metric>max [vision] of agents</metric>
    <metric>max [metabolic-rate] of agents</metric>
    <metric>max [sugar-stock] of agents</metric>
    <metric>max [spice-stock] of agents</metric>
    <metric>standard-deviation [vision] of agents</metric>
    <metric>standard-deviation [metabolic-rate] of agents</metric>
    <metric>standard-deviation [sugar-stock] of agents</metric>
    <metric>median [age] of agents</metric>
    <metric>median [length children] of agents</metric>
    <enumeratedValueSet variable="Scenario">
      <value value="&quot;0: No change&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-Population">
      <value value="400"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-Location">
      <value value="&quot;Anywhere&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Growback-Method">
      <value value="&quot;Fixed-Rate&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Sugar-Growback-Rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Spice-Growback-Rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Spice?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Trade?">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Seasons?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Season-Length">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Off-Season-Inter-Growback-Time">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Endowment">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Endowment">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Vision">
      <value value="3.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Metabolic-Rate">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Foresight">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-Age-0?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Death-Age">
      <value value="60"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Death-Age">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Reproduction">
      <value value="&quot;Sexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Fertility-Start">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Fertility-Start">
      <value value="15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Fertility-End-Male">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Fertility-End-Female">
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Fertility-End-Male">
      <value value="60"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Fertility-End-Female">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Inheritance?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Culture?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Number-Of-Tags">
      <value value="11"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Number-Of-Groups">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Combat?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Lending?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Loan-Period">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Interest-Rate">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution-Diffusion-Period">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution-On-At-Time">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution-Diffusion-On-At">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Production-Pollution">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Consumption-Pollution">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Disease?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Number-Of-Diseases">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Disease-Effect">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Disease-Code-Length">
      <value value="3.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Agents-Initial-Number-Of-Diseases">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Immune-System-Length">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Agent-Color-Scheme">
      <value value="&quot;Age&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Patch-Color-Scheme">
      <value value="&quot;Sugar &amp; Spice&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Link-Parentage?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Hide-Loan-Links?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Print-Trades?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="RNG-Seed">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-ticks">
      <value value="1000"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="experiment-Loans" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>timer</metric>
    <metric>count agents</metric>
    <metric>gini</metric>
    <metric>mean [vision] of agents</metric>
    <metric>mean [metabolic-rate] of agents</metric>
    <metric>mean [sugar-stock] of agents</metric>
    <metric>mean [spice-stock] of agents</metric>
    <metric>median [sugar-stock] of agents</metric>
    <metric>median [spice-stock] of agents</metric>
    <metric>min [vision] of agents</metric>
    <metric>min [metabolic-rate] of agents</metric>
    <metric>min [sugar-stock] of agents</metric>
    <metric>min [spice-stock] of agents</metric>
    <metric>max [vision] of agents</metric>
    <metric>max [metabolic-rate] of agents</metric>
    <metric>max [sugar-stock] of agents</metric>
    <metric>max [spice-stock] of agents</metric>
    <metric>standard-deviation [vision] of agents</metric>
    <metric>standard-deviation [metabolic-rate] of agents</metric>
    <metric>standard-deviation [sugar-stock] of agents</metric>
    <metric>median [age] of agents</metric>
    <metric>median [length children] of agents</metric>
    <metric>(geomean trade-prices)</metric>
    <metric>(length trade-prices)</metric>
    <metric>(standard-deviation (map [log ? 10] trade-prices))</metric>
    <enumeratedValueSet variable="Scenario">
      <value value="&quot;0: No change&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-Population">
      <value value="400"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-Location">
      <value value="&quot;Anywhere&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Growback-Method">
      <value value="&quot;Fixed-Rate&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Sugar-Growback-Rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Spice-Growback-Rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Spice?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Trade?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Seasons?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Season-Length">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Off-Season-Inter-Growback-Time">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Endowment">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Endowment">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Vision">
      <value value="3.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Metabolic-Rate">
      <value value="3"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Foresight">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-Age-0?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Death-Age">
      <value value="60"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Death-Age">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Reproduction">
      <value value="&quot;Sexual&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Fertility-Start">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Fertility-Start">
      <value value="15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Fertility-End-Male">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Fertility-End-Female">
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Fertility-End-Male">
      <value value="60"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Fertility-End-Female">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Inheritance?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Culture?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Number-Of-Tags">
      <value value="11"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Number-Of-Groups">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Combat?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Lending?">
      <value value="false"/>
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Loan-Period">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Interest-Rate">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution-Diffusion-Period">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution-On-At-Time">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution-Diffusion-On-At">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Production-Pollution">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Consumption-Pollution">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Disease?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Number-Of-Diseases">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Disease-Effect">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Disease-Code-Length">
      <value value="3.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Agents-Initial-Number-Of-Diseases">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Immune-System-Length">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Agent-Color-Scheme">
      <value value="&quot;Age&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Patch-Color-Scheme">
      <value value="&quot;Sugar &amp; Spice&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Link-Parentage?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Hide-Loan-Links?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Print-Trades?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="RNG-Seed">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-ticks">
      <value value="1000"/>
    </enumeratedValueSet>
  </experiment>
  <experiment name="Fig-II-5" repetitions="10" runMetricsEveryStep="false">
    <setup>setup</setup>
    <go>go</go>
    <metric>timer</metric>
    <metric>count agents</metric>
    <metric>gini</metric>
    <metric>mean [vision] of agents</metric>
    <metric>mean [metabolic-rate] of agents</metric>
    <metric>mean [sugar-stock] of agents</metric>
    <metric>median [sugar-stock] of agents</metric>
    <metric>min [vision] of agents</metric>
    <metric>min [metabolic-rate] of agents</metric>
    <metric>min [sugar-stock] of agents</metric>
    <metric>max [vision] of agents</metric>
    <metric>max [metabolic-rate] of agents</metric>
    <metric>max [sugar-stock] of agents</metric>
    <metric>median [age] of agents</metric>
    <enumeratedValueSet variable="Initial-Population">
      <value value="500"/>
    </enumeratedValueSet>
    <steppedValueSet variable="Mean-Vision" first="1" step="1" last="10.5"/>
    <steppedValueSet variable="Mean-Metabolic-Rate" first="1" step="0.5" last="3"/>
    <enumeratedValueSet variable="Max-Resources">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Loan-Period">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Death-Age">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Fertility-Start">
      <value value="12"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Immune-System-Length">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Endowment">
      <value value="25"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Fertility-End-Female">
      <value value="40"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Production-Pollution">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Fertility-End-Male">
      <value value="60"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Death-Age">
      <value value="1000"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution-On-At-Time">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Number-Of-Groups">
      <value value="2"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-ticks">
      <value value="500"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Culture?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Number-Of-Diseases">
      <value value="20"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Endowment">
      <value value="5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Seasons?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Hide-Loan-Links?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Consumption-Pollution">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Spice-Growback-Rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Season-Length">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-Location">
      <value value="&quot;Anywhere&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Scenario">
      <value value="&quot;1: Movement (Ch. 2, p.30-1)&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Lending?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution-Diffusion-On-At">
      <value value="100"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Patch-Color-Scheme">
      <value value="&quot;Sugar&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Link-Parentage?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Reproduction">
      <value value="&quot;None&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Foresight">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Sugar-Growback-Rate">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Grid-Width">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Disease-Effect">
      <value value="0.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Off-Season-Inter-Growback-Time">
      <value value="8"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Mean-Disease-Code-Length">
      <value value="3.5"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="RNG-Seed">
      <value value="0"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Print-Trades?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Agents-Initial-Number-Of-Diseases">
      <value value="4"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Disease?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Number-Of-Tags">
      <value value="11"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Trade?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Min-Fertility-End-Male">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Inheritance?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Fertility-End-Female">
      <value value="50"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Initial-Age-0?">
      <value value="true"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Growback-Method">
      <value value="&quot;Fixed-Rate&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Interest-Rate">
      <value value="10"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Spice?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Pollution-Diffusion-Period">
      <value value="1"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Agent-Color-Scheme">
      <value value="&quot;Red&quot;"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Max-Fertility-Start">
      <value value="15"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Combat?">
      <value value="false"/>
    </enumeratedValueSet>
    <enumeratedValueSet variable="Scape-Type">
      <value value="&quot;Classic: 2 hills of Sugar &amp; 2 hills of Spice&quot;"/>
    </enumeratedValueSet>
  </experiment>
</experiments>
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
